/*
Available at http://www.koders.com/c/fid9237D3E4242B19DCA0CBAAFF18C751C5E5D8576C.aspx
originally from the HMMER package for Maximum Likelihood Fitting of Extreme Value Distributions

  @misc{ Eddy97,
  author = "Eddy, S. R.",
  title = "Maximum Likelihood Fitting of Extreme Value Distributions",
  url = "citeseer.ist.psu.edu/370503.html",
  YEAR = {1997}, }

*/


#include <stdlib.h>
#include <stdio.h>
#include "generic.h"


void* ckalloc(size_t bytes)
{
  void* ret;
  	
  if( (ret = calloc(bytes, sizeof(char))) == NULL) {
		printf("Out of memory, on asking for %d bytes\n",bytes);
		return NULL; 	/* for the moment, could fail here */
	}else{  
		return ret;
	}  
}



void* ckcalloc(int len, size_t bytes)
{
	return ckalloc(len * bytes);
}



void* ckrealloc(void *ptr, size_t bytes)
{
  void *ret;

  if (ptr == NULL) {	
		printf("Bad call to ckrealloc, NULL pointer\n");
		return NULL;
	}else if( (ret = realloc(ptr, bytes)) == NULL) {
		printf("Out of memory, trying to realloc %d bytes\n",bytes);
		return NULL;
	}else{
		return ret;	
	}
}


void* ckfree(void* ptr)
{

  if (ptr == NULL){
    printf("Bad call to ckfree - NULL pointer\n");
  }else{
    free(ptr);
    ptr = NULL;
  }
  return ptr;
}

/* fatal - print message and die */
void fatal(const char* msg)
{
	fprintf(stderr, "%s\n", msg);
	exit(1);
}

/* fatalf - format message, print it, and die */
void fatalf(const char* msg, char* val)
{
	fprintf(stderr, msg, val);
	putc('\n', stderr);
	exit(1);
}
	


/* ckalloc1 - allocate space; check for success */
void *ckalloc1(size_t  amount)
{
	void* p;

	if ( (p = malloc(amount) ) == NULL)
		fatal("Ran out of memory\n.");
	return (p);
}


void iniStr(char* s, int len)
{
	int i;
	for(i = 0; i < len; i++) {
		s[i]='\0';
	}
}


 

 /* 
 Functon : GetProperThreadNum()
 *	Descrip: compute the number of threads that should launch for a give 'for loop'
 *	based on the whole number of threads and the number of CPU cores 
 *  the maximum threads does not exceed the number of CPU cores.
 
     @param   int n - the whole iterations    
     @param   int min_n - the minimum iteration of each threads    
     @return int - number of threads that is suggestted to set     
*/
 
int GetProperThreadNum(int iters, int min_n)
{

   int max_tn = iters / min_n;
   int threadNum = max_tn > G_NumCore ? G_NumCore : max_tn; 
   
   if ( threadNum < 1 ) threadNum = 1;
   
   return threadNum;
}

