#ifndef MY_ERROR_HANDLER_H
#define MY_ERROR_HANDLER_H 

#include <stdexcept> 
#include <string> 
#include <mpi.h>

#include <exception>
#include <string>
static const char* memErrMsg = "Memory allocation fails\n";
static const char* ErrOpenFile = "Can not open file\n";
#define MY_THROW(ExClass, args...)                             	\
    do                                                         	\
     {                                                          \
         ExClass e(args);                                       \
         e.Init(__FILE__, __PRETTY_FUNCTION__, __LINE__);       \
         throw e;                                               \
     }                                                          \
	 while (false);     
  
#define MY_DEFINE_EXCEPTION(ExClass, Base)                     	\
     ExClass(const std::string& msg = "") throw()               \
         : Base(msg)                                            \
     {}                                                         \
                                                                \
     ~ExClass() throw() {}                                      \
                                                                \
	std::string GetClassName() const            				\
     {                                                          \
         return #ExClass;                                       \
     }

class ExceptionBase : public std::exception
{
	public:
		ExceptionBase(const std::string& msg = "") throw();

		virtual ~ExceptionBase() throw();

		void Init(const char* file, const char* func, int line);

		virtual std::string GetClassName() const;

		virtual std::string GetMessage() const;

		const char* what() const throw();

		const std::string& ToString() const;

		std::string GetStackTrace() const;

	protected:
		std::string mMsg;
		const char* mFile;
		const char* mFunc;
		int mLine;

	private:
		enum { MAX_STACK_TRACE_SIZE = 50 };
		void* mStackTrace[MAX_STACK_TRACE_SIZE];
		size_t mStackTraceSize;
		mutable std::string mWhat;
};

class ExceptionDerived : public ExceptionBase
{
	public:
		MY_DEFINE_EXCEPTION(ExceptionDerived, ExceptionBase);
};




 namespace MYErrorHandler {
 //subclass so we can specifically catch MPI errors 
 
	class Exception : public std::exception 
	{   
		public:     
			Exception(std::string const& what) : std::exception(), m_what(what) { }     
			virtual ~Exception() throw() { }     
			virtual const char* what() const throw() 
			{       
				return m_what.c_str( );     
			}    
		protected:     
			std::string m_what;   
	};
	 
	void convertToException( MPI_Comm *comm, int *err, ... ); 

}  

#endif

