#ifndef _PAIRWISE_
#define _PAIRWISE_
#include <InputParse.h>
#include <smith_waterman.h>
const static int  Nmax 	= 3000;
const static int  Qmax	= 100;
const static int  Lmax	= 4;
const static int  Cmax	= 7;
const static int  topPvalue = 100;


const static int MaxSeqLen		= 3000;
const static int MaxMulAlign	= 500;
const static int MaxPathLen 	= 1024;
const static int MaxNameLen 	= 100;


typedef struct tagSeqInfo{
	int length;
	char name[MaxNameLen];
	char context[MaxSeqLen];
	int cls[4]; //information about classification
}SeqInfo;

typedef struct SeqEntrytag{
	int index;
	int length;	
}SeqEntry;

typedef struct pairwiseRlt {
	int index1;
	char name1[MaxNameLen];
	int index2;
	char name2[MaxNameLen];
	int topolog;
	int homolog;
	double lambda;
	double K;
	double score;
	double pvalue;
} pairwiseRlt;

class StatSig {

public:
	StatSig(CInputParse* params);
	~StatSig();
		
	void 	run();
	void 	GetPSSE(int* localSubIndex);
	
	void 	GenerateScores_inter(int shuffles_ML, int* PSSM, char* querySeq,
						   char* shuffleSeq,int queryLen, int subLen,
						   int* Scores
						   );

	void 	StatSigForSeqPair_ML(int seqLen1,int seqLen2,
							double& lambda_ML_censor,double& K_ML_censor,
							double& pwAlgnScore,double& pwStatSig,
							int* Scores
							); 
	void 	ShuffleSeqs( char* seq, int len, int seeds);
	void 	PrintResult();
	void 	ReadSeqs();
	int 	loadClsOfSeqs();
	void 	ReadPSSMFromFile(char* PSSMFilename, int queryLen);
	void 	writePSSToFile (pairwiseRlt* PSSInfo, int num, char* EvalueScvFile);
	//for quick sort
	static int compar_ascent_pvalue(const void * va, const void * vb);
	static int compar_descent_pvalue(const void* va, const void* vb);
	static int compar_ascent_length(const void * va, const void * vb);
	static int compar_descent_length(const void* va, const void* vb);

private:
	
	double 	getSysTime();
	void 	iniStr(char* s, int n);
	int 	FindStringIndex(char* SeqName, SeqInfo* dbSeqs, int dbSeqNum);
	int** 	dbSeqClf;
	void 	iniMallocMem();
	// sorted by qsort alg.
	SeqEntry* sortedSubSeqs;
	SeqEntry* sortedQuerySeqs;
	
	int splitNum;  /* each node hanle the number of query on average */
	
	int restNum; /* the number of subject sequences all the nodes work on together */
	
	int localSubNum;
	int* globalSubNum;	
	int* globalSubIndex;
		
	pairwiseRlt* globalPSSEInfo;
	
	double 
			totalTime, IOTime, sfl_align_Time,
			sflTime, alnTime, fitTime;
		
	double 	lambda_ML_censor, 	//returned statistical parameter lambda
			K_ML_censor, 		//returned statistical parameter K
			pwAlgnScore, 		//returned pairwise alignment score
			pwStatSig; 			//returned pairwise statistical significance

	
	// Pointer of resource Files
	FILE *fp_DB, *fp_querySeq, *fp_EPQ;
	// query and subject sequences information
	SeqInfo* DBSeqs;
	SeqInfo* QuerySeqs;

	int gapOpen;
	int gapExten;
	
	int TotalSeqs; // total number of subject sequences
	int TotalQuery; // total number of query sequences
	int shuffles_ML; //number of shuffling time	

	
	pairwiseRlt* PSSInfo;	
	char* DBFile; 
	char* seqQueryFile; 
	char* seqClsFile;
		
	char* PSSMFilePath; //just path, no specific file
	int* PSSMatrix; // PSS Matrix
	
	int* amino_acids_trans;
	char* amino_acids;
	
	int nProcessors; /* no. of processors */
	int	pRank;		 /* rank of processor */
	
	int 	para_type; /* inter- or intra- parallelism */
	int 	threads; 	/* the number of threads in parallel */
	CInputParse* params; /* parsing inputs of users */
};

// set size of acids amino acid residue is 23
static  char amino_acids_23[23] = {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
    'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R',
    'S', 'T', 'V', 'W', 'X', 'Y', 'Z'
};

static int amino_acids_trans_23[256] = {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1,  0,  1,  2,  3,  4,  5,  6,  7,  8, -1,  9, 10, 11, 12, -1,
    13, 14, 15, 16, 17, -1, 18, 19, 20, 21, 22, -1, -1, -1, -1, -1,
    -1,  0,  1,  2,  3,  4,  5,  6,  7,  8, -1,  9, 10, 11, 12, -1,
    13, 14, 15, 16, 17, -1, 18, 19, 20, 21, 22, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

//here the substitutions for 20 amino acids
static  int amino_acids_trans_20[256] = {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1,  0, -1,  1,  2,  3,  4,  5,  6,  7, -1,  8,  9, 10, 11, -1,
    12, 13, 14, 15, 16, -1, 17, 18, -1, 19, -1, -1, -1, -1, -1, -1,
    -1,  0, -1,  1,  2,  3,  4,  5,  6,  7, -1,  8,  9, 10, 11, -1,
    12, 13, 14, 15, 16, -1, 17, 18, -1, 19, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

// set size of acids amino acid residue is 20
static  char amino_acids_20[20] = {
    'A', 'C', 'D', 'E', 'F', 'G', 'H',
    'I', 'K', 'L', 'M', 'N', 'P', 'Q', 'R',
    'S', 'T', 'V', 'W', 'Y'
};
			
#endif

