/***********************************************
* #
* # GPL 2.0 applies.
* #
* # Editor :Zhang Yuhong
* # zhangyuhong001@gmail.com
* # 2011.5
* ************************************************/


#ifndef _GENEIC_H_
#define _GENEIC_H_
#include <stddef.h>
#include <err.h>
#include <omp.h>
#define DEBUG
static int		MIN_ITRA_NUM 	= 4; /* minimum iteration number */
static const int G_NumCore 		= omp_get_num_procs(); //obtain the number the CPU cores

#define USE_MPI 1
#define MIN(a,b) ((a)<(b) ? (a) : (b))
#define MAX(a,b) ((a)>(b) ? (a) : (b))

//to check here if work? this macro can help reduce and lock the error space.
#define IFWORK { printf("Work here: %s, line  %d \n", __FILE__ ,__LINE__); }
#define CHECK(x) { cout <<__FILE__<< "  "<<__LINE__<<": the variable checked '" <<#x <<"' = "<<x<<endl;}   	

#ifdef DEBUG

#define ERROR_OUT(reason) \
	do{ { \
				fprintf(stdout, "\n%s, line  %d, %s\n",__FILE__ ,__LINE__, reason); \
				fflush(stdout); \
				return;\
	}}while(0);

#else

#define ERROR_OUT(reason) \
	do{ { \
				fprintf(stdout, "\n%s\n", reason); \
				fflush(stdout); \
				return;\
	}}while(0);

#endif

/* 
 Functon : GetProperThreadNum()
 *	Descrip: compute the number of threads that should launch for a give 'for loop'
 *	based on the whole number of threads and the number of CPU cores 
 *  the maximum threads does not exceed the number of CPU cores.
 
     @param   int iters - the whole iterations    
     @param   int min_n - the minimum iteration of each threads    
     @return int - number of threads that is suggestted to set     
*/


int GetProperThreadNum(int iters, int min_n);


	
/* Function:  ckalloc(bytes)
 *
 * Descrip:    Tries to alloc bytes of memory. Posts
 *             to warn if it fails
 *
 *
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckalloc(size_t bytes);



/* Function:  ckcalloc(len,bytes)
 *
 * Descrip:    calloc equivalent
 *
 *
 * Arg:          len [UNKN ] Undocumented argument [int]
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckcalloc(int len,size_t bytes);



/* Function:  ckrealloc(*ptr,bytes)
 *
 * Descrip:    realloc equivalent
 *
 *
 * Arg:         *ptr [UNKN ] Undocumented argument [void]
 * Arg:        bytes [UNKN ] Undocumented argument [size_t]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckrealloc(void *ptr, size_t bytes);



/* Function:  ckfree(*ptr)
 *
 * Descrip:    free equivalent
 *
 *
 * Arg:        *ptr [UNKN ] Undocumented argument [void]
 *
 * Return [UNKN ]  Undocumented return value [void *]
 *
 */
void* ckfree(void *ptr);

void iniStr(char* s, int len);

/* fatal - print message and die */
void fatal(const char* msg);


/* fatalf - format message, print it, and die */
void fatalf(const char* msg, char* val);
	
/* ckalloc1 - allocate space; check for success */
void *ckalloc1(size_t  amount);


#endif


