/***********************************************
* #
* # GPL 2.0 applies.
* #
* # written by  Zhang Yuhong using C++
* # zhangyuhong001@gmail.com
* # 2011.5
* ************************************************/

#ifndef _SMITH_WATERMAN_H
#define _SMITH_WATERMAN_H

static const int  	maxQueryLen 	= 5000; 
static const int   	maxSubjectLen 	= 30760;  /* maximum sequence lengths */
static int			MAX_AMINO_ACIDS = 23;		


typedef struct tagUshort2
{ 
	unsigned short int x; 
	unsigned short int y; 
} ushort2;

typedef struct tagIntraContext
{
	// middle results for intra-task parallism	
	int H1[maxQueryLen]; // H (i-1, j-1)
	int H2[maxQueryLen]; // H (i-1,j)
	int H3[maxQueryLen]; // current H(i,j)
	int FF[maxQueryLen]; // current F(i,j)
	int E1[maxQueryLen]; // E(i-1,j)
	int E2[maxQueryLen]; // E(i, j)

}IntraContext;

class Alinment
{
public:
	Alinment();
	~Alinment();
	void iniSettings(int gap, int gapExt, 
					int* trans_table,
					int* PSSMatrix);
					
	int  smith_waterman_inter(char* Seq1, char* Seq2,  	/* sequences */
						int  Seq1Len, int Seq2Len  	/* their lengths */
						); 
	
	int  smith_waterman_intra(char* Seq1, char* Seq2,  	/* sequences */
						int  Seq1Len, int Seq2Len  	/* their lengths */						
								
						);
						
private:
	
	int seq1Len, seq2Len;
	char* Seq1;
	char* Seq2;
	
	IntraContext midRlt;
	
	int Gap; 			/* gap penalties */
	int GapExtend; 		/* gap extend penalties */
	int GapOE;			/* gap penalties + gap extend penalties */		
	int* Matrix; 		/* substitution scores matrix */
	int* amino_acids_trans;	
	int anti_diag_len;
};


#endif

