/**********************************************************************
* # GPL 2.0 applies.
* # Author  :Zhang Yuhong
* # Email:	zhangyuhong001@gmail.com
* # Filename: smith_waterman.cpp
* # Function:  Alinment (query, subject) pairwise sequence 
				using smith_waterman algorithm
* # Latestd updated 05/21/2011	
***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <generic.h>
#include <iostream>
#include <smith_waterman.h>
#include <omp.h>
#include <exception>
#include <cxxabi.h>
#include <iostream>
#include <sstream>
#include <execinfo.h>
#include <ErrorHandle.h>
#include <iostream>

using namespace std;

Alinment::Alinment()
{
	Seq1 				= NULL;
	Seq2 				= NULL;	
	Matrix 				= NULL;
	amino_acids_trans 	= NULL;
	
}

Alinment::~Alinment()
{
	// do nothing
}
						

void Alinment::iniSettings(int gap, int gapExt,
						   int* trans_table, int* PSSMatrix)
{
	Gap 		= gap; 				/* gap penalties */
	GapExtend 	= gapExt; 			/* gap extend penalties */
	GapOE 		= Gap + GapExtend;	/* gap penalties + gap extend penalties */
	
	if ( Gap < 0 )
	{
	  MY_THROW(ExceptionDerived, "Error: The gap-open penalty is a nonnegative integer");
	}
	if ( GapExtend < 0 )
	{
	   MY_THROW(ExceptionDerived, "Error: The gap-extend penalty is a nonnegative integer");
	}
	
	amino_acids_trans 	= trans_table;
	Matrix				= PSSMatrix;
	
	
}

// smith_waterman inter-task parallism
int Alinment::smith_waterman_inter(char* Seq1, char* Seq2,  	/* sequences */
						int  Seq1Len, int Seq2Len  		/* their lengths */						
						)
{ 

	if ( Seq1Len == 0 && Seq2Len == 0 )
	{
	   MY_THROW(ExceptionDerived, "Error: One of sequence are empty!");
	}else if (Seq1Len != strlen(Seq1) || Seq2Len != strlen(Seq2)){
		MY_THROW(ExceptionDerived,"Error: One of sequence format is wrong!");
	}
	

  	if ( Seq1Len > maxQueryLen || Seq2Len > maxSubjectLen )
	{	
		MY_THROW(ExceptionDerived, "Error: Sequences are too long");
	}
	
	seq1Len = Seq1Len;
	seq2Len = Seq2Len;
	
	ushort2* MidRlt = (ushort2*) calloc(Seq2Len, sizeof (ushort2));
	if ( false == MidRlt )
	{
		 MY_THROW(ExceptionDerived, "Error: Memory allocation falis!");
	}
	
	int H, P, F, E;
	E = 0;
	H = 0;
	int  MaxScore = 0;   		/* the max score in LIST */
	char sc; 					//subject sequence character
	int subMatScore = 0;		//local profile query substition matrix score
	int dicIndex = 0; 			// The lexicographicall index of residues set 
	
	/* 
	HD :temporary middle results
	HD.x : H(i -1, j) --> left cell
	HD.y : E(i -1, j) --> left cell
	*/
	ushort2 HD;	

	for (int queryLen = 0; queryLen < Seq1Len; queryLen++) // query: outter loop
	{
		
		P = 0; // the diagonal H(i-1,j-1)
		F = 0; //the above F(i-1,j)

		for (int subLen = 0; subLen < Seq2Len; subLen++)// subject : inner loop
		{

			sc 			= Seq2[ subLen ];
			dicIndex 	= amino_acids_trans [ sc ];
			//load PSSM matrix
			subMatScore = Matrix[dicIndex * Seq1Len + queryLen];

			// read middle results
			HD = MidRlt[ subLen ];

			// serially compute the score of the cell (0, 0), cell (0, 1),cell (0, 2),...
			// then serially compute the score of the cell (1, 0), cell (1, 1),cell (1, 2), ...
			// and so on...
			F = max (H - GapOE, F - GapExtend);
			E = max (HD.x - GapOE, HD.y - GapExtend);
			
			H = P + subMatScore; // update H
			H = max( H , F);
			H = max( H , E);
			H = max( H , 0);
			
			MaxScore  = max( MaxScore, H);
			
			// update diagonal H(i-1,j-1)
			P 	= 	HD.x;			
			HD.x 	= min(H, 0x0FFFF);
			E 		= max(E, 0);
			HD.y 	= min(E, 0x0FFFF);
			
			// update middle result
			MidRlt [ subLen ] = HD;
			
		}

	}
	
	// release memory of middle results 
	if (MidRlt){
		free(MidRlt);
		MidRlt = NULL;
	}
	return 	MaxScore;
}

// smith_waterman intra-task parallism
int  Alinment::smith_waterman_intra(char* Seq1, char* Seq2,  	/* sequences */
									int  Seq1Len, int Seq2Len  	/* their lengths */
									)
{
	if ( Seq1Len == 0 && Seq2Len == 0 )
	{
	   MY_THROW(ExceptionDerived,"Error: One of sequence are empty!");
	}else if (Seq1Len != strlen(Seq1) || Seq2Len != strlen(Seq2)){
		MY_THROW(ExceptionDerived,"Error: One of sequence format is wrong!");
	}
	

  	if ( Seq1Len > maxQueryLen || Seq2Len > maxSubjectLen )
	{	
		MY_THROW(ExceptionDerived,"Error: Sequences are too long");
	}

	int Min_Len 	= MIN(Seq1Len, Seq2Len);

	/* use local variable to instore the context of intra-task */
	int* H1 = midRlt.H1;
	int* H2 = midRlt.H2;
	int* H3 = midRlt.H3;
	int* FF = midRlt.FF;
	int* E1 = midRlt.E1;
	int* E2 = midRlt.E2;

	
	/* initialize the middle results as zero */
	memset(H1, 0, maxQueryLen * sizeof(int));
	memset(H2, 0, maxQueryLen * sizeof(int));
	memset(H3, 0, maxQueryLen * sizeof(int));
	memset(FF, 0, maxQueryLen * sizeof(int));
	memset(E1, 0, maxQueryLen * sizeof(int));
	memset(E2, 0, maxQueryLen * sizeof(int));
	
	int dicIndex;
	int start_sub_index; // the first residue of subject sequence to be processed.
	int score = 0;
	int h, f, e; // the 3 temp value of each cell
	char sc;	// residues of subject sequence
	int subMatScore;

	for(int i = 1; i <= Seq1Len + Seq2Len - 1; i++ )
	{
		
		start_sub_index = i;
		int start_inner = 0;
		int end_inner 	= min(min(i, Seq1Len + Seq2Len - i), Min_Len);
		
		if (i > Seq2Len){
			start_sub_index 	= Seq2Len;
			start_inner 		= i - Seq2Len;
		}
		
		end_inner += start_inner;
		

		for(int j = start_inner; j < end_inner; j++)
		{		

			//calculate the vetical value : F
			f = max(FF[ j ] - GapExtend, H2[ j ] - GapOE);
			f = max(f, 0);
			
			//calcuate the horizontal value : E
			e = max(E1[ j - 1 ] - GapExtend, H2[ j - 1 ] - GapOE);
			e = max(e, 0);

			//calculate the diagonal value : H
			sc 		= Seq2[ start_sub_index - 1 ];
			dicIndex 	= amino_acids_trans [ sc ];			
			//load PSSM matrix
			subMatScore = Matrix [ dicIndex * Seq1Len +  j ];
			h = H1[ j - 1 ] + subMatScore;	
			h = max(h, e);
			h = max(h, f);
			h = min(h, 0x0FFFF);

			//save the values
			H3[ j ] = h;
			FF[ j ] = min(f, 0x0FFFF);
			E2[ j ] = min(e, 0x0FFFF);
		
			score = max(score, h);
			
			start_sub_index--;

		} //end for the inner loop : j
		
		//swap the buffers A <- B; B <- C;
		int* tmp = H1;
		H1 = H2;
		H2 = H3;
		H3 = tmp;

		tmp = E2;
		E2 	= E1;
		E1 	= tmp;

	} //end for the outter loop : i
	
	return score;
}	
